==> rnsf7.74.04_src.zip: LvCsv.cpp <==
#include "stdafx.h"
#include "Rnsf7.h"
#include "lvcsv.h"
#include "opendlg.h"

// mean̔z񐔂JEg
int lvCountColumn(const char **means)
{
	int nColumn;

	for(nColumn=0 ; means[nColumn] ; ++nColumn)
		;

	return nColumn;
}


// Xgr[̓eCSVt@C֏o͂
// ̃[ɏ]B
// EZ̓J}ŋ؂
// EZɃJ}_uR[eBV܂܂Ăꍇ́A_uR[eBVň͂
// EZɃ_uR[eBV܂܂Ăꍇ́A_uR[eBVd˂
// EZɉsR[h܂܂Ăꍇ́A߂ȂȂ|x
void lvSaveToCsv(HWND hWnd, HWND hListView, int nColumn)
{
	char fname[MAX_PATH*2] = "";
	int n, i, c;
	char *pstr;

	if ( saveDialog( hWnd, "CSVt@C(*.csv)\0*.csv\0ׂẴt@C(*.*)\0*.*\0", fname, _countof(fname), "CSVt@C֕ۑ", "csv" ) ) {
		FILE *fp;
		int  iErr = 0;

		if (0 == fopen_s(&fp, fname, "wt")) {
			n = ListView_GetItemCount(hListView);
			for(i=0 ; i<n ; ++i) {
				for(c=0 ; c<nColumn ; ++c) {
					// J}؂̎
					if (c > 0) {
						if (EOF == fputc(',', fp)) {
							iErr = errno;
							break;
						}
					}
					pstr = lvGetItemString(hListView, i, c);
					if (pstr) {
						if (strpbrk(pstr, ",\"\r\n") == NULL) {
							// "  , s܂܂ĂȂꍇ́A_uR[e[Vň͂܂Ȃ
							if (EOF == fputs(pstr, fp)) {
								iErr = errno;
								break;
							}
						}
						else {
							// "  , s܂܂Ăꍇ́A_uR[e[Vň͂(J)
							if (EOF == fputc('"', fp)) {
								iErr = errno;
								break;
							}

							for(int j=0 ; pstr[j] ; ++j) {
								// ePo
								if (EOF == fputc(pstr[j], fp)) {
									iErr = errno;
									break;
								}

								// "̏ꍇ2dɂďo
								if (pstr[j] == '"') {
									if (EOF == fputc(pstr[j], fp)) {
										iErr = errno;
										break;
									}
								}
							}
							if (iErr) {
								break;
							}

							// "  , s܂܂Ăꍇ́A_uR[e[Vň͂()
							if (EOF == fputc('"', fp)) {
								iErr = errno;
								break;
							}
						}
						safefree(pstr);
					}
				}
				if (iErr) {
					break;
				}
				if (EOF == fputc('\n', fp)) {
					iErr = errno;
					break;
				}
			}
			fclose(fp);

			if (iErr) {
				DeleteFile(fname);

				char mes[1025];
#pragma warning( disable : 4996 )
				wsprintf(mes, "%s ֏݂ł܂łB\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
				MessageBox(hWnd, mes, "CSVt@C֏o", MB_OK | MB_ICONSTOP);
			}
			else {
				MessageBox(hWnd, "CSVt@Cւ̏o͂s܂B", "CSVt@C֏o", MB_OK | MB_ICONINFORMATION);
			}
		}
		else {
			iErr = errno;

			char mes[1025];
#pragma warning( disable : 4996 )
			wsprintf(mes, "%s J܂B\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
			MessageBox(hWnd, mes, "CSVt@C֏o", MB_OK | MB_ICONSTOP);
		}
	}
}

// Xgr[̓eCSVǂݎ鋤ʊ֐
// ̃[ɏ]B
// EZ̓J}ŋ؂
// EZɃJ}_uR[eBV܂܂Ăꍇ́A_uR[eBVň͂
// EZɃ_uR[eBV܂܂Ăꍇ́A_uR[eBVd˂
// Es̓rɉsꍇAȍ~̃J͋ƂȂB
// EPs̃JXgr[̃JꍇA͖B
// EPJڂɏd̂śAx\舵ȂB
// EPs̃TCY2KB܂ŁB
bool lvLoadFromCsvCom(HWND hWnd, HWND hListView, int nColumn, bool bClear)
{
	char fname[MAX_PATH*2] = "";

	if ( openDialog( hWnd, "CSVt@C(*.csv)\0*.csv\0ׂẴt@C(*.*)\0*.*\0", fname, _countof(fname), "CSVt@Cǂݏo", "csv" ) ) {
		FILE *fp;
		int  iErr = 0;
		bool bChofuku = 0;

		if (0 == fopen_s(&fp, fname, "rt")) {
			LVITEM lvitem;
			char buf[2050], szPart[2050], *pPart;
			int  c, j, new_i = 0, k;

			if (bClear) {
				// OɃNA
				ListView_DeleteAllItems(hListView);
			}

			while(!feof(fp)) {
				ZeroMemory(buf, sizeof(buf));

				// 1sǂݍ
				if (NULL == fgets(buf, _countof(buf)-1, fp)) {
					if (!feof(fp)) {
						iErr = errno;
					}
					break;
				}

				// 1s̉
				for(c=j=0 ; c<nColumn && buf[j] ; ++c) {
					
					if (buf[j] == '"') {
						// _uR[e[VŎn܂Ăꍇ c ӂēǂݏo
						pPart =  szPart;
						ZeroMemory(szPart, sizeof(szPart));
						k = 0;
						++j;

						for(k=0 ;; ++j) {
							if (buf[j] == '"' && buf[j+1] == '"') {
								// " + " = "
								szPart[k] = '"';
								++j;
								++k;
							}
							else if (buf[j] == '"' && buf[j+1] == ',') {
								// " + , = ,
								j += 2;
								++k;
								break;
							}
							else if (buf[j] == '"' && (buf[j+1] == '\r' || buf[j+1] == '\n' || buf[j+1] == '\0')) {
								// " + sI[
								break;
							}
							else if (buf[j] == '\0') {
								// Ȃ肠ꂽI[ c ̍s֑s
								// 1sǂݍ
								if (NULL == fgets(buf, _countof(buf)-1, fp)) {
									if (!feof(fp)) {
										iErr = errno;
									}
									break;
								}
								j = -1;
							}
							else {
								szPart[k] = buf[j];
								++k;
							}
						}

						if (iErr) {
							break;
						}
					}
					else {
						// _uR[e[VŎn܂ĂȂꍇ c J}siI[j܂œǂݏo
						pPart = &buf[j];
						while(!(buf[j] == ',' || buf[j] == '\r' || buf[j] == '\n' || buf[j] == '\0'))
							++j;

						if (buf[j] == ',') {
							buf[j] = '\0';
							++j;
						}
						else if (buf[j] == '\r' || buf[j] == '\n') {
							buf[j] = '\0';
						}
					}

					if(c == 0) {
						// ŏ̃JȂCT[g

						// PJɏdx
						char *pstr;
						bool bChofuku1 = 0;
						int n = ListView_GetItemCount(hListView);
						for(int i=0 ; i<n ; ++i) {
							pstr = lvGetItemString(hListView, i, 0);
							if (pstr) {
								if ( _stricmp(pPart, pstr) == 0 ) {
									free(pstr);
									bChofuku1 = 1;
									break;
								}
								free(pstr);
							}
						}

						if (bChofuku1) {
							bChofuku = 1;
							break; // dXLbv
						}
						
						// CT[g
						ZeroMemory(&lvitem, sizeof(LVITEM));
						lvitem.mask = LVIF_TEXT;
						lvitem.iItem = n;
						lvitem.iSubItem = 0;
						lvitem.pszText = pPart;
						new_i = ListView_InsertItem(hListView, &lvitem);
					}
					else {
						// łȂ΃Zbg
						ListView_SetItemText(hListView, new_i, c, pPart);
					}
				}
			}

			fclose(fp);

			if (iErr) {
				char mes[1025];
#pragma warning( disable : 4996 )
				wsprintf(mes, "%s ǂݍ݂ł܂łB\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
				MessageBox(hWnd, mes, "CSVt@C", MB_OK | MB_ICONSTOP);
			}
			else if (bChofuku) {
				MessageBox(hWnd, "CSVt@C͂s܂B\r\n1Jɏd̂f[^͏O܂B", "CSVt@C", MB_OK | MB_ICONWARNING);
			}
			else {
				MessageBox(hWnd, "CSVt@C͂s܂B", "CSVt@C", MB_OK | MB_ICONINFORMATION);
			}

			return 1;
		}
		else {
			iErr = errno;

			char mes[1025];
#pragma warning( disable : 4996 )
			wsprintf(mes, "%s J܂B\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
			MessageBox(hWnd, mes, "CSVt@C", MB_OK | MB_ICONSTOP);

			return 0;
		}
	}
	return 0;
}


// Xgr[̓eCSVt@C͂Bɂf[^͎B
bool lvLoadFromCsv(HWND hWnd, HWND hListView, int nColumn)
{
	return lvLoadFromCsvCom(hWnd, hListView, nColumn, 1);
}

// Xgr[̓eCSVt@Cǉ͂Bɂf[^͎ȂB
bool lvAppendFromCsv(HWND hWnd, HWND hListView, int nColumn)
{
	return lvLoadFromCsvCom(hWnd, hListView, nColumn, 0);
}

==> rnsf7.75.07_src.zip: LvCsv.cpp <==
#include "stdafx.h"
#include "Rnsf7.h"
#include "lvcsv.h"
#include "opendlg.h"

// mean̔z񐔂JEg
int lvCountColumn(const char **means)
{
	int nColumn;

	for(nColumn=0 ; means[nColumn] ; ++nColumn)
		;

	return nColumn;
}


// Xgr[̓eCSVt@C֏o͂
// ̃[ɏ]B
// EZ̓J}ŋ؂
// EZɃJ}_uR[e[V܂܂Ăꍇ́A_uR[e[Vň͂
// EZɃ_uR[e[V܂܂Ăꍇ́A_uR[e[Vd˂
// EZɉsR[h܂܂Ăꍇ́A߂ȂȂ|x
void lvSaveToCsv(HWND hWnd, HWND hListView, int nColumn)
{
	char fname[MAX_PATH*2] = "";
	int n, i, c;
	char *pstr;

	if ( saveDialog( hWnd, "CSVt@C(*.csv)\0*.csv\0ׂẴt@C(*.*)\0*.*\0", fname, _countof(fname), "CSVt@C֕ۑ", "csv" ) ) {
		FILE *fp;
		int  iErr = 0;

		if (0 == fopen_s(&fp, fname, "wt")) {
			n = ListView_GetItemCount(hListView);
			for(i=0 ; i<n ; ++i) {
				for(c=0 ; c<nColumn ; ++c) {
					// J}؂̎
					if (c > 0) {
						if (EOF == fputc(',', fp)) {
							iErr = errno;
							break;
						}
					}
					pstr = lvGetItemString(hListView, i, c);
					if (pstr) {
						if (strpbrk(pstr, ",\"\r\n") == NULL) {
							// "  , s܂܂ĂȂꍇ́A_uR[e[Vň͂܂Ȃ
							if (EOF == fputs(pstr, fp)) {
								iErr = errno;
								break;
							}
						}
						else {
							// "  , s܂܂Ăꍇ́A_uR[e[Vň͂(J)
							if (EOF == fputc('"', fp)) {
								iErr = errno;
								break;
							}

							for(int j=0 ; pstr[j] ; ++j) {
								// ePo
								if (EOF == fputc(pstr[j], fp)) {
									iErr = errno;
									break;
								}

								// "̏ꍇ2dɂďo
								if (pstr[j] == '"') {
									if (EOF == fputc(pstr[j], fp)) {
										iErr = errno;
										break;
									}
								}
							}
							if (iErr) {
								break;
							}

							// "  , s܂܂Ăꍇ́A_uR[e[Vň͂()
							if (EOF == fputc('"', fp)) {
								iErr = errno;
								break;
							}
						}
						safefree(pstr);
					}
				}
				if (iErr) {
					break;
				}
				if (EOF == fputc('\n', fp)) {
					iErr = errno;
					break;
				}
			}
			fclose(fp);

			if (iErr) {
				DeleteFile(fname);

				char mes[1025];
#pragma warning( disable : 4996 )
				wsprintf(mes, "%s ֏݂ł܂łB\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
				MessageBox(hWnd, mes, "CSVt@C֏o", MB_OK | MB_ICONSTOP);
			}
			else {
				MessageBox(hWnd, "CSVt@Cւ̏o͂s܂B", "CSVt@C֏o", MB_OK | MB_ICONINFORMATION);
			}
		}
		else {
			iErr = errno;

			char mes[1025];
#pragma warning( disable : 4996 )
			wsprintf(mes, "%s J܂B\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
			MessageBox(hWnd, mes, "CSVt@C֏o", MB_OK | MB_ICONSTOP);
		}
	}
}

// Xgr[̓eCSVǂݎ鋤ʊ֐
// ̃[ɏ]B
// EZ̓J}ŋ؂
// EZɃJ}_uR[e[V܂܂Ăꍇ́A_uR[e[Vň͂
// EZɃ_uR[e[V܂܂Ăꍇ́A_uR[e[Vd˂
// Es̓rɉsꍇAȍ~̃J͋ƂȂB
// EPs̃JXgr[̃JꍇA͖B
// EPJڂɏd̂śAx\舵ȂB
// EPs̃TCY2KB܂ŁB
bool lvLoadFromCsvCom(HWND hWnd, HWND hListView, int nColumn, bool bClear)
{
	char fname[MAX_PATH*2] = "";

	if ( openDialog( hWnd, "CSVt@C(*.csv)\0*.csv\0ׂẴt@C(*.*)\0*.*\0", fname, _countof(fname), "CSVt@Cǂݏo", "csv" ) ) {
		FILE *fp;
		int  iErr = 0;
		bool bChofuku = 0;

		if (0 == fopen_s(&fp, fname, "rt")) {
			LVITEM lvitem;
			char buf[2050], szPart[2050], *pPart;
			int  c, j, new_i = 0, k;

			if (bClear) {
				// OɃNA
				ListView_DeleteAllItems(hListView);
			}

			while(!feof(fp)) {
				ZeroMemory(buf, sizeof(buf));

				// 1sǂݍ
				if (NULL == fgets(buf, _countof(buf)-1, fp)) {
					if (!feof(fp)) {
						iErr = errno;
					}
					break;
				}

				// 1s̉
				for(c=j=0 ; c<nColumn && buf[j] ; ++c) {
					
					if (buf[j] == '"') {
						// _uR[e[VŎn܂Ăꍇ c ӂēǂݏo
						pPart =  szPart;
						ZeroMemory(szPart, sizeof(szPart));
						k = 0;
						++j;

						for(k=0 ;; ++j) {
							if (buf[j] == '"' && buf[j+1] == '"') {
								// " + " = "
								szPart[k] = '"';
								++j;
								++k;
							}
							else if (buf[j] == '"' && buf[j+1] == ',') {
								// " + , = ,
								j += 2;
								++k;
								break;
							}
							else if (buf[j] == '"' && (buf[j+1] == '\r' || buf[j+1] == '\n' || buf[j+1] == '\0')) {
								// " + sI[
								break;
							}
							else if (buf[j] == '\0') {
								// Ȃ肠ꂽI[ c ̍s֑s
								// 1sǂݍ
								if (NULL == fgets(buf, _countof(buf)-1, fp)) {
									if (!feof(fp)) {
										iErr = errno;
									}
									break;
								}
								j = -1;
							}
							else {
								szPart[k] = buf[j];
								++k;
							}
						}

						if (iErr) {
							break;
						}
					}
					else {
						// _uR[e[VŎn܂ĂȂꍇ c J}siI[j܂œǂݏo
						pPart = &buf[j];
						while(!(buf[j] == ',' || buf[j] == '\r' || buf[j] == '\n' || buf[j] == '\0'))
							++j;

						if (buf[j] == ',') {
							buf[j] = '\0';
							++j;
						}
						else if (buf[j] == '\r' || buf[j] == '\n') {
							buf[j] = '\0';
						}
					}

					if(c == 0) {
						// ŏ̃JȂCT[g

						// PJɏdx
						char *pstr;
						bool bChofuku1 = 0;
						int n = ListView_GetItemCount(hListView);
						for(int i=0 ; i<n ; ++i) {
							pstr = lvGetItemString(hListView, i, 0);
							if (pstr) {
								if ( _stricmp(pPart, pstr) == 0 ) {
									free(pstr);
									bChofuku1 = 1;
									break;
								}
								free(pstr);
							}
						}

						if (bChofuku1) {
							bChofuku = 1;
							break; // dXLbv
						}
						
						// CT[g
						ZeroMemory(&lvitem, sizeof(LVITEM));
						lvitem.mask = LVIF_TEXT;
						lvitem.iItem = n;
						lvitem.iSubItem = 0;
						lvitem.pszText = pPart;
						new_i = ListView_InsertItem(hListView, &lvitem);
					}
					else {
						// łȂ΃Zbg
						ListView_SetItemText(hListView, new_i, c, pPart);
					}
				}
			}

			fclose(fp);

			if (iErr) {
				char mes[1025];
#pragma warning( disable : 4996 )
				wsprintf(mes, "%s ǂݍ݂ł܂łB\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
				MessageBox(hWnd, mes, "CSVt@C", MB_OK | MB_ICONSTOP);
			}
			else if (bChofuku) {
				MessageBox(hWnd, "CSVt@C͂s܂B\r\n1Jɏd̂f[^͏O܂B", "CSVt@C", MB_OK | MB_ICONWARNING);
			}
			else {
				MessageBox(hWnd, "CSVt@C͂s܂B", "CSVt@C", MB_OK | MB_ICONINFORMATION);
			}

			return 1;
		}
		else {
			iErr = errno;

			char mes[1025];
#pragma warning( disable : 4996 )
			wsprintf(mes, "%s J܂B\r\n%s", fname, _sys_errlist[iErr]);
#pragma warning( default : 4996 )
			MessageBox(hWnd, mes, "CSVt@C", MB_OK | MB_ICONSTOP);

			return 0;
		}
	}
	return 0;
}


// Xgr[̓eCSVt@C͂Bɂf[^͎B
bool lvLoadFromCsv(HWND hWnd, HWND hListView, int nColumn)
{
	return lvLoadFromCsvCom(hWnd, hListView, nColumn, 1);
}

// Xgr[̓eCSVt@Cǉ͂Bɂf[^͎ȂB
bool lvAppendFromCsv(HWND hWnd, HWND hListView, int nColumn)
{
	return lvLoadFromCsvCom(hWnd, hListView, nColumn, 0);
}
